@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
 #include <signal.h>
 #include <grp.h>
 #include <pwd.h>
@@ -31,13 +32,12 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gio/gio.h>
 
 #include "update-notifier.h"
-#include "hal.h"
 #include "update.h"
 #include "hooks.h"
+#include "gdu.h"
 #include "reboot.h"
 #include "crash.h"
 #include "avahi.h"
@@ -55,9 +55,9 @@ gboolean update_timer_finished(gpointer data);
 
 // the timeout (in sec) when a further activity from dpkg/apt
 // causes the applet to "ungray"
-#define TIMEOUT_APT_RUN 120 /* 120 sec */
+#define TIMEOUT_APT_RUN 600 /* 600 sec */
 
-// force statup even if the user is not in the admin group
+// force startup even if the user is not in the admin group
 static gboolean FORCE_START=FALSE;
 
 // force gksu for all menu actions
@@ -95,23 +95,49 @@ g_debug_inotify(const char *msg, ...)
    va_end(va);
 }
 
-void invoke(gchar *cmd, gchar *desktop, gboolean with_gksu)
+void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
 {
-   if(with_gksu || FORCE_GKSU)
+   GdkAppLaunchContext *context;
+   GAppInfo *appinfo;
+   GError *error = NULL;
+   static GtkWidget *w = NULL;
+
+   // gksu
+   if(with_gksu || FORCE_GKSU) {
       invoke_with_gksu(cmd, desktop, FALSE);
-   else
-      g_spawn_command_line_async(cmd, NULL);
+      return;
+   }
+
+   // fake window to get the current server time *urgh*
+   if (!w) {
+      w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      gtk_widget_realize (w);
+   }
+
+   // normal launch
+   context = gdk_app_launch_context_new ();
+   guint32 timestamp =  gdk_x11_get_server_time (w->window);
+   appinfo = g_app_info_create_from_commandline(cmd, 
+						cmd, 
+						G_APP_INFO_CREATE_NONE,
+						&error);
+   gdk_app_launch_context_set_timestamp (context, timestamp);
+   if (!g_app_info_launch (appinfo, NULL, context, &error))
+      g_warning ("Launching failed: %s\n", error->message);
+   g_object_unref (context);
+   g_object_unref (appinfo);
+
 }
 
 void
-invoke_with_gksu(gchar *cmd, gchar *descr, gboolean whole_message)
+invoke_with_gksu(const gchar *cmd, const gchar *descr, gboolean whole_message)
 {
         //g_print("invoke_update_manager ()\n");
-	gchar *argv[5];
+        gchar *argv[5];
 	argv[0] = "/usr/bin/gksu";
 	argv[1] = whole_message ? "--message" : "--desktop";
-	argv[2] = descr;
-	argv[3] = cmd;
+	argv[2] = (gchar*)descr;
+	argv[3] = (gchar*)cmd;
 	argv[4] = NULL;
 
 	g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, NULL);
@@ -295,17 +321,20 @@ gboolean file_monitor_periodic_check(gpointer data)
  * End user might not understand the message at all */
 void u_abort(gchar *msg)
 {
-   msg = g_strdup_printf("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n", _("Internal error"), msg);
-   gtk_dialog_run(GTK_DIALOG(gtk_message_dialog_new_with_markup(NULL,0,
+   const char *fmt = "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n";
+   gtk_dialog_run(GTK_DIALOG(gtk_message_dialog_new_with_markup(NULL, 0,
 						     GTK_MESSAGE_ERROR,
 						     GTK_BUTTONS_CLOSE,
+						     fmt,
+						     _("Internal error"),
 						     msg)));
    g_free(msg);
    exit(1);
 }
 
 // FIXME: get the apt directories with apt-config or something
-gboolean monitor_init(UpgradeNotifier *un)
+gboolean 
+monitor_init(UpgradeNotifier *un)
 {
    int i;
    GFileMonitor *monitor_handle;
@@ -318,11 +347,11 @@ gboolean monitor_init(UpgradeNotifier *un)
       CRASHREPORT_DIR,
       NULL};
    for(i=0;monitor_dirs[i] != NULL;i++) {
-      GError err;
+      GError *error = NULL;
       GFile *gf = g_file_new_for_path(monitor_dirs[i]);
-      monitor_handle = g_file_monitor_directory(gf, 0, NULL, &err);
+      monitor_handle = g_file_monitor_directory(gf, 0, NULL, &error);
       if(monitor_handle)
-	 g_signal_connect(monitor_handle, "changed", monitor_cb, un);
+	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
       else
 	 g_warning("can not add '%s'\n", monitor_dirs[i]);
    }
@@ -336,11 +365,11 @@ gboolean monitor_init(UpgradeNotifier *un)
       UNICAST_LOCAL_AVAHI_FILE,
       NULL};
    for(i=0;monitor_files[i] != NULL;i++) {
-      GError err;
+      GError *error = NULL;
       GFile *gf = g_file_new_for_path(monitor_files[i]);
-      monitor_handle = g_file_monitor_file(gf, 0, NULL, &err);
+      monitor_handle = g_file_monitor_file(gf, 0, NULL, &error);
       if(monitor_handle)
-	 g_signal_connect(monitor_handle, "changed", monitor_cb, un);
+	 g_signal_connect(monitor_handle, "changed", (GCallback)monitor_cb, un);
       else
 	 g_warning("can not add '%s'\n", monitor_dirs[i]);
    }
@@ -449,25 +478,24 @@ static GOptionEntry entries[] =
    { "debug-inotify", 0, 0, G_OPTION_ARG_NONE, &INOTIFY_DEBUG, "Enable inotify debugging"},
    { "force", 0, 0, G_OPTION_ARG_NONE, &FORCE_START, "Force start even if the user is not in the admin group"},
    { "force-use-gksu", 0, 0, G_OPTION_ARG_NONE, &FORCE_GKSU, "Force running all commands (update-manager, synaptic) with gksu" },
-   { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay statup by given amount of seconds" },
+   { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay startup by given amount of seconds" },
    { NULL }
 };
 
 int 
-main (int argc, char *argv[])
+main (int argc, char **argv)
 {
-	GnomeClient *client;
 	UpgradeNotifier *un;
+	GError *error = NULL;
 
-	// option parsing
-	GOptionContext *context;
-	context = g_option_context_new (" - inform about updates");
-	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	gnome_program_init (PACKAGE, PACKAGE_VERSION, 
-			    LIBGNOMEUI_MODULE,
-			    argc, argv, 
-			    GNOME_PARAM_GOPTION_CONTEXT, context,
-			    GNOME_PARAM_NONE);
+	// init
+	if(!gtk_init_with_args (&argc, &argv, 
+				_("- inform about updates"), entries, 
+				"update-notifier", &error) ) {
+	   fprintf(stderr, _("Failed to init the UI: %s\n"), 
+		   error ? error->message : _("unknown error"));
+	   exit(1);
+	}
 
 	notify_init("update-notifier");
         bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
@@ -511,22 +539,19 @@ main (int argc, char *argv[])
 
 	// delay icon creation for 30s so that the desktop 
 	// login is not slowed down by update-notifier
-	g_timeout_add_seconds(STARTUP_DELAY, (GSourceFunc)(tray_icons_init), un);
+	g_timeout_add_seconds(STARTUP_DELAY, 
+			      (GSourceFunc)(tray_icons_init), un);
 
         // initial check for avahi
         avahi_disabled_check();
-
-	// init hal (needs to be done _after_ the icons are created)
-	/* setup hal so that inserted cdroms can be checked */
-	LibHalContext *hal_ctx = up_do_hal_init();
-	if (!hal_ctx) {
-	   u_abort("failed to initialize HAL!\n");
-	   return 1;
-	}
-	libhal_ctx_set_user_data(hal_ctx, un);
 	
-	// init fam
-	monitor_init(un);
+	/* setup hal so that inserted cdroms can be checked */
+	if(!up_do_hal_init(un)) {
+	   g_warning("initializing gdu failed");
+	}
+	
+	// init gio file monitoring
+	monitor_init (un);
 	
 	/* Start the main gtk loop */
 	gtk_main ();
